@@ -14,7 +14,7 @@ import pytest
 
 from botocore.client import ClientEndpointBridge
 from botocore.exceptions import NoRegionError
-from tests import BaseSessionTest, ClientHTTPStubber, create_session, mock
+from tests import BaseSessionTest, ClientHTTPStubber, mock
 
 # NOTE: sqs endpoint updated to be the CN in the SSL cert because
 # a bug in python2.6 prevents subjectAltNames from being parsed
@@ -61,7 +61,7 @@ KNOWN_REGIONS = {
         'streams.dynamodb': 'streams.dynamodb.ap-northeast-1.amazonaws.com',
         'sts': 'sts.ap-northeast-1.amazonaws.com',
         'swf': 'swf.ap-northeast-1.amazonaws.com',
-        'workspaces': 'workspaces.ap-northeast-1.amazonaws.com'
+        'workspaces': 'workspaces.ap-northeast-1.amazonaws.com',
     },
     'ap-southeast-1': {
         'autoscaling': 'autoscaling.ap-southeast-1.amazonaws.com',
@@ -93,7 +93,7 @@ KNOWN_REGIONS = {
         'streams.dynamodb': 'streams.dynamodb.ap-southeast-1.amazonaws.com',
         'sts': 'sts.ap-southeast-1.amazonaws.com',
         'swf': 'swf.ap-southeast-1.amazonaws.com',
-        'workspaces': 'workspaces.ap-southeast-1.amazonaws.com'
+        'workspaces': 'workspaces.ap-southeast-1.amazonaws.com',
     },
     'ap-southeast-2': {
         'autoscaling': 'autoscaling.ap-southeast-2.amazonaws.com',
@@ -128,11 +128,9 @@ KNOWN_REGIONS = {
         'streams.dynamodb': 'streams.dynamodb.ap-southeast-2.amazonaws.com',
         'sts': 'sts.ap-southeast-2.amazonaws.com',
         'swf': 'swf.ap-southeast-2.amazonaws.com',
-        'workspaces': 'workspaces.ap-southeast-2.amazonaws.com'
-    },
-    'aws-us-gov-global': {
-        'iam': 'iam.us-gov.amazonaws.com'
+        'workspaces': 'workspaces.ap-southeast-2.amazonaws.com',
     },
+    'aws-us-gov-global': {'iam': 'iam.us-gov.amazonaws.com'},
     'cn-north-1': {
         'autoscaling': 'autoscaling.cn-north-1.amazonaws.com.cn',
         'cloudformation': 'cloudformation.cn-north-1.amazonaws.com.cn',
@@ -155,7 +153,7 @@ KNOWN_REGIONS = {
         'storagegateway': 'storagegateway.cn-north-1.amazonaws.com.cn',
         'streams.dynamodb': 'streams.dynamodb.cn-north-1.amazonaws.com.cn',
         'sts': 'sts.cn-north-1.amazonaws.com.cn',
-        'swf': 'swf.cn-north-1.amazonaws.com.cn'
+        'swf': 'swf.cn-north-1.amazonaws.com.cn',
     },
     'eu-central-1': {
         'autoscaling': 'autoscaling.eu-central-1.amazonaws.com',
@@ -185,7 +183,7 @@ KNOWN_REGIONS = {
         'storagegateway': 'storagegateway.eu-central-1.amazonaws.com',
         'streams.dynamodb': 'streams.dynamodb.eu-central-1.amazonaws.com',
         'sts': 'sts.eu-central-1.amazonaws.com',
-        'swf': 'swf.eu-central-1.amazonaws.com'
+        'swf': 'swf.eu-central-1.amazonaws.com',
     },
     'eu-west-1': {
         'apigateway': 'apigateway.eu-west-1.amazonaws.com',
@@ -229,14 +227,10 @@ KNOWN_REGIONS = {
         'streams.dynamodb': 'streams.dynamodb.eu-west-1.amazonaws.com',
         'sts': 'sts.eu-west-1.amazonaws.com',
         'swf': 'swf.eu-west-1.amazonaws.com',
-        'workspaces': 'workspaces.eu-west-1.amazonaws.com'
-    },
-    'fips-us-gov-west-1': {
-        's3': 's3-fips.us-gov-west-1.amazonaws.com'
-    },
-    's3-external-1': {
-        's3': 's3-external-1.amazonaws.com'
+        'workspaces': 'workspaces.eu-west-1.amazonaws.com',
     },
+    'fips-us-gov-west-1': {'s3': 's3-fips.us-gov-west-1.amazonaws.com'},
+    's3-external-1': {'s3': 's3-external-1.amazonaws.com'},
     'sa-east-1': {
         'autoscaling': 'autoscaling.sa-east-1.amazonaws.com',
         'cloudformation': 'cloudformation.sa-east-1.amazonaws.com',
@@ -260,7 +254,7 @@ KNOWN_REGIONS = {
         'storagegateway': 'storagegateway.sa-east-1.amazonaws.com',
         'streams.dynamodb': 'streams.dynamodb.sa-east-1.amazonaws.com',
         'sts': 'sts.sa-east-1.amazonaws.com',
-        'swf': 'swf.sa-east-1.amazonaws.com'
+        'swf': 'swf.sa-east-1.amazonaws.com',
     },
     'us-east-1': {
         'apigateway': 'apigateway.us-east-1.amazonaws.com',
@@ -316,7 +310,7 @@ KNOWN_REGIONS = {
         'support': 'support.us-east-1.amazonaws.com',
         'swf': 'swf.us-east-1.amazonaws.com',
         'workspaces': 'workspaces.us-east-1.amazonaws.com',
-        'waf': 'waf.amazonaws.com'
+        'waf': 'waf.amazonaws.com',
     },
     'us-gov-west-1': {
         'autoscaling': 'autoscaling.us-gov-west-1.amazonaws.com',
@@ -338,7 +332,7 @@ KNOWN_REGIONS = {
         'sns': 'sns.us-gov-west-1.amazonaws.com',
         'sqs': 'us-gov-west-1.queue.amazonaws.com',
         'sts': 'sts.us-gov-west-1.amazonaws.com',
-        'swf': 'swf.us-gov-west-1.amazonaws.com'
+        'swf': 'swf.us-gov-west-1.amazonaws.com',
     },
     'us-west-1': {
         'autoscaling': 'autoscaling.us-west-1.amazonaws.com',
@@ -368,7 +362,7 @@ KNOWN_REGIONS = {
         'storagegateway': 'storagegateway.us-west-1.amazonaws.com',
         'streams.dynamodb': 'streams.dynamodb.us-west-1.amazonaws.com',
         'sts': 'sts.us-west-1.amazonaws.com',
-        'swf': 'swf.us-west-1.amazonaws.com'
+        'swf': 'swf.us-west-1.amazonaws.com',
     },
     'us-west-2': {
         'apigateway': 'apigateway.us-west-2.amazonaws.com',
@@ -412,8 +406,8 @@ KNOWN_REGIONS = {
         'streams.dynamodb': 'streams.dynamodb.us-west-2.amazonaws.com',
         'sts': 'sts.us-west-2.amazonaws.com',
         'swf': 'swf.us-west-2.amazonaws.com',
-        'workspaces': 'workspaces.us-west-2.amazonaws.com'
-    }
+        'workspaces': 'workspaces.us-west-2.amazonaws.com',
+    },
 }
 
 
@@ -427,19 +421,10 @@ KNOWN_AWS_PARTITION_WIDE = {
     'route53': 'https://route53.amazonaws.com',
     's3': 'https://s3.amazonaws.com',
     'sts': 'https://sts.amazonaws.com',
-    'iam': 'https://iam.amazonaws.com'
+    'iam': 'https://iam.amazonaws.com',
 }
 
 
-def _get_patched_session():
-    with mock.patch('os.environ') as environ:
-        environ['AWS_ACCESS_KEY_ID'] = 'access_key'
-        environ['AWS_SECRET_ACCESS_KEY'] = 'secret_key'
-        environ['AWS_CONFIG_FILE'] = 'no-exist-foo'
-        session = create_session()
-    return session
-
-
 def _known_endpoints_by_region():
     for region_name, service_dict in KNOWN_REGIONS.items():
         for service_name, endpoint in service_dict.items():
@@ -448,17 +433,18 @@ def _known_endpoints_by_region():
 
 @pytest.mark.parametrize(
     "service_name, region_name, expected_endpoint",
-    _known_endpoints_by_region()
+    _known_endpoints_by_region(),
 )
-def test_single_service_region_endpoint(service_name, region_name, expected_endpoint):
+def test_single_service_region_endpoint(
+    patched_session, service_name, region_name, expected_endpoint
+):
     # Verify the actual values from the partition files.  While
     # TestEndpointHeuristics verified the generic functionality given any
     # endpoints file, this test actually verifies the partition data against a
     # fixed list of known endpoints.  This list doesn't need to be kept 100% up
     # to date, but serves as a basis for regressions as the endpoint data
     # logic evolves.
-    resolver = _get_patched_session()._get_internal_component(
-        'endpoint_resolver')
+    resolver = patched_session._get_internal_component('endpoint_resolver')
     bridge = ClientEndpointBridge(resolver, None, None)
     result = bridge.resolve(service_name, region_name)
     expected = 'https://%s' % expected_endpoint
@@ -466,8 +452,8 @@ def test_single_service_region_endpoint(service_name, region_name, expected_endp
 
 
 # Ensure that all S3 regions use s3v4 instead of v4
-def test_all_s3_endpoints_have_s3v4():
-    session = _get_patched_session()
+def test_all_s3_endpoints_have_s3v4(patched_session):
+    session = patched_session
     partitions = session.get_available_partitions()
     resolver = session._get_internal_component('endpoint_resolver')
     for partition_name in partitions:
@@ -478,28 +464,26 @@ def test_all_s3_endpoints_have_s3v4():
 
 
 @pytest.mark.parametrize(
-    "service_name, expected_endpoint",
-    KNOWN_AWS_PARTITION_WIDE.items()
+    "service_name, expected_endpoint", KNOWN_AWS_PARTITION_WIDE.items()
 )
-def test_single_service_partition_endpoint(service_name, expected_endpoint):
-    resolver = _get_patched_session()._get_internal_component(
-        'endpoint_resolver')
+def test_single_service_partition_endpoint(
+    patched_session, service_name, expected_endpoint
+):
+    resolver = patched_session._get_internal_component('endpoint_resolver')
     bridge = ClientEndpointBridge(resolver)
     result = bridge.resolve(service_name)
     assert result['endpoint_url'] == expected_endpoint
 
 
-def test_non_partition_endpoint_requires_region():
-    resolver = _get_patched_session()._get_internal_component(
-        'endpoint_resolver')
+def test_non_partition_endpoint_requires_region(patched_session):
+    resolver = patched_session._get_internal_component('endpoint_resolver')
     with pytest.raises(NoRegionError):
         resolver.construct_endpoint('ec2')
 
 
 class TestEndpointResolution(BaseSessionTest):
-
     def setUp(self):
-        super(TestEndpointResolution, self).setUp()
+        super().setUp()
         self.xml_response = (
             b'<?xml version="1.0" encoding="UTF-8"?>\n\n'
             b'<ListRolesResponse '
@@ -509,7 +493,9 @@ class TestEndpointResolution(BaseSessionTest):
         )
 
     def create_stubbed_client(self, service_name, region_name, **kwargs):
-        client = self.session.create_client(service_name, region_name, **kwargs)
+        client = self.session.create_client(
+            service_name, region_name, **kwargs
+        )
         http_stubber = ClientHTTPStubber(client)
         http_stubber.start()
         return client, http_stubber
@@ -519,8 +505,7 @@ class TestEndpointResolution(BaseSessionTest):
         stubber.add_response()
         client.list_buckets()
         self.assertEqual(
-            stubber.requests[0].url,
-            'https://s3.us-east-2.amazonaws.com/'
+            stubber.requests[0].url, 'https://s3.us-east-2.amazonaws.com/'
         )
 
     def test_regionalized_client_with_unknown_region(self):
@@ -530,8 +515,7 @@ class TestEndpointResolution(BaseSessionTest):
         # Validate we don't fall back to partition endpoint for
         # regionalized services.
         self.assertEqual(
-            stubber.requests[0].url,
-            'https://s3.not-real.amazonaws.com/'
+            stubber.requests[0].url, 'https://s3.not-real.amazonaws.com/'
         )
 
     def test_unregionalized_client_endpoint_resolution(self):
@@ -549,3 +533,15 @@ class TestEndpointResolution(BaseSessionTest):
         self.assertTrue(
             stubber.requests[0].url.startswith('https://iam.amazonaws.com/')
         )
+
+
+@pytest.mark.parametrize("is_builtin", [True, False])
+def test_endpoint_resolver_knows_its_datasource(patched_session, is_builtin):
+    # The information whether or not the endpoints.json file was loaded from
+    # the builtin data directory or not should be passed from Loader to
+    # EndpointResolver.
+    session = patched_session
+    loader = session.get_component('data_loader')
+    with mock.patch.object(loader, 'is_builtin_path', return_value=is_builtin):
+        resolver = session._get_internal_component('endpoint_resolver')
+        assert resolver.uses_builtin_data == is_builtin
